# Notification Server

A Drupal module that provides real-time notification capabilities through both HTTP and WebSocket protocols.

## Description

The Notification Server module enables real-time notifications in Drupal applications using two different approaches:
1. HTTP-based notifications with server-sent events
2. WebSocket-based real-time communication

This module is particularly useful for:

- Real-time updates in web applications
- Push notifications to specific channels
- Live data streaming
- Chat applications
- Real-time dashboards

## Features

- **HTTP Notifications**
  - Server-sent events (SSE) support
  - Channel-based message publishing
  - Simple HTTP API for sending notifications

- **WebSocket Support**
  - Real-time bidirectional communication
  - Channel subscription management
  - Message broadcasting to specific channels
  - Connection status monitoring

- **Administration**
  - Configuration interface for server settings
  - Demo interfaces for both HTTP and WebSocket notifications
  - Permission-based access control

## Requirements

- Drupal 10.x
- PHP 8.1 or higher
- WebSocket server (e.g., Ratchet, ReactPHP)

## Installation

1. Download and place the module in your Drupal installation:
   ```bash
   cd web/modules/custom
   git clone [repository-url] notification_server
   ```

2. Enable the module using Drush:
   ```bash
   drush en notification_server
   ```

3. Configure the module at: Administration » Configuration » System » Notification Server

## Configuration

1. Visit `/admin/config/system/notification-server`
2. Configure the following settings:
   - WebSocket server URL
   - HTTP notification settings
   - Security settings

## Documentation

### Service Documentation

- [Notification Server Client Service](docs/NotificationServerClient.md) - Detailed documentation for using the `notification_server.client` service
- [HTTP Notification API](docs/HttpNotificationApi.md) - HTTP-based notification API documentation
- [WebSocket Integration](docs/WebSocketIntegration.md) - WebSocket integration guide

## Permissions

The module provides the following permissions:
- `administer notification_server configuration`: Access to module configuration
- `access notification_server http demo`: Access to HTTP notification demo
- `access notification_server websocket demo`: Access to WebSocket demo interface

## Quick Start

### Using the Notification Client Service

```php
// Get the notification client service
$notification_client = \Drupal::service('notification_server.client');

// Send a simple notification
$notification_client->publish('channel_name', 'Hello, World!');

// Send a structured notification
$notification_client->publish('user_notifications', [
  'type' => 'info',
  'message' => 'New content available',
  'data' => [
    'contentId' => 123,
    'title' => 'Updated Article',
  ],
]);
```

For more detailed examples and advanced usage, see the [Notification Server Client Service Documentation](docs/NotificationServerClient.md).

### WebSocket Integration

1. Connect to WebSocket server:
   ```javascript
   const socket = new WebSocket('ws://your-websocket-server:port');
   ```

2. Subscribe to a channel:
   ```javascript
   socket.send(JSON.stringify({
     type: 'subscribe',
     channel: 'channel_name',
     clientId: 'unique_client_id'
   }));
   ```

3. Listen for messages:
   ```javascript
   socket.onmessage = function(event) {
     const data = JSON.parse(event.data);
     console.log('Received:', data);
   };
   ```

## Demo Interfaces

1. HTTP Notification Demo:
   - URL: `/admin/config/system/notification-server/http-demo`
   - Demonstrates HTTP-based notifications
   - Allows testing message publishing

2. WebSocket Demo:
   - URL: `/admin/config/system/notification-server/websocket-demo`
   - Shows real-time WebSocket communication
   - Provides channel subscription interface
   - Displays connection status

## Troubleshooting

1. **Connection Issues**
   - Verify WebSocket server is running
   - Check server URL configuration
   - Ensure proper firewall settings

2. **Permission Problems**
   - Verify user has required permissions
   - Check Drupal logs for access issues

3. **Message Delivery**
   - Confirm channel names are correct
   - Check WebSocket server logs
   - Verify client is subscribed to channel

## Contributing

1. Follow Drupal.org contrib module contributing practices.

## License

This project is licensed under the GNU General Public License v2.0 or later.

## Support

- Issue Queue: [Project Issues](https://www.drupal.org/project/issues/notification_server)
- Documentation: [Project Documentation](https://www.drupal.org/docs/8/modules/notification_server)
- Chat: [Drupal Slack #notification_server](https://drupal.slack.com/messages/notification_server)

## Maintainers

- [D34dMan] - Initial work and maintenance

## Acknowledgments

- Drupal Community
- WebSocket protocol specifications
- Server-Sent Events (SSE) standard 