<?php

declare(strict_types=1);

namespace Drupal\notification_server\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\notification_server\DTO\ChannelDTO;
use Drupal\notification_server\DTO\ChannelRulesDTO;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\HttpFoundation\Response;

/**
 * Service for interacting with the Notification Server API.
 */
class NotificationServerClient implements NotificationServerClientInterface
{

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a new NotificationServerClient object.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory.
   */
  public function __construct(
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('notification_server');
  }

  /**
   * Get the base server URL from configuration.
   *
   * @return string
   *   The server URL.
   */
  protected function getServerUrl(): string {
    return rtrim($this->configFactory->get('notification_server.settings')->get('server_url'), '/');
  }

  /**
   * Publish a new notification.
   *
   * @param string $channel
   *   The channel to publish the notification to.
   * @param string $message
   *   The notification message content.
   *
   * @return array|null
   *   The notification data if successful, NULL otherwise.
   */
  public function publishNotification(string $channel, string $message): ?array {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . '/api/notifications', [
        'json' => [
          'channel' => $channel,
          'message' => $message,
        ],
      ]);

      if ($response->getStatusCode() === Response::HTTP_OK) {
        return json_decode($response->getBody()->getContents(), TRUE);
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to publish notification: @error', ['@error' => $e->getMessage()]);
    }

    return NULL;
  }

  /**
   * Get notification history for a channel.
   *
   * @param string $channel
   *   The channel name.
   * @param int $limit
   *   Maximum number of notifications to return (default: 10).
   *
   * @return array
   *   Array of notifications.
   */
  public function getNotifications(string $channel, int $limit = 10): array {
    try {
      $response = $this->httpClient->request('GET', $this->getServerUrl() . '/api/notifications/' . $channel, [
        'query' => [
          'limit' => max(1, min($limit, 100)), // Ensure limit is between 1 and 100
        ],
      ]);

      if ($response->getStatusCode() === Response::HTTP_OK) {
        return json_decode($response->getBody()->getContents(), TRUE) ?? [];
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to get notifications: @error', ['@error' => $e->getMessage()]);
    }

    return [];
  }

  /**
   * Subscribe a client to a channel.
   *
   * @param string $channel
   *   The channel name to subscribe to.
   * @param string $clientId
   *   The WebSocket client ID.
   *
   * @return bool
   *   TRUE if subscription was successful, FALSE otherwise.
   */
  public function subscribeToChannel(string $channel, string $clientId): bool {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . '/api/channels/' . $channel . '/subscribe', [
        'json' => [
          'clientId' => $clientId,
        ],
      ]);

      return $response->getStatusCode() === Response::HTTP_OK;
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to subscribe to channel: @error', ['@error' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Unsubscribe a client from a channel.
   *
   * @param string $channel
   *   The channel name to unsubscribe from.
   * @param string $clientId
   *   The WebSocket client ID.
   *
   * @return bool
   *   TRUE if unsubscription was successful, FALSE otherwise.
   */
  public function unsubscribeFromChannel(string $channel, string $clientId): bool {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . '/api/channels/' . $channel . '/unsubscribe', [
        'json' => [
          'clientId' => $clientId,
        ],
      ]);

      return $response->getStatusCode() === Response::HTTP_OK;
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to unsubscribe from channel: @error', ['@error' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Generate a new client ID.
   * 
   * @param string $client_id
   *   The client ID to validate/generate.
   *
   * @param array $metadata
   *   Optional metadata for the client (e.g., userAgent, environment).
   *
   * @return string|null
   *   The generated client ID if successful, NULL otherwise.
   */
  public function generateClientId(string $client_id = NULL,  array $metadata = []): ?string {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . '/api/clients', [
        'json' => [
          'clientId' => $client_id,
          'metadata' => $metadata,
        ],
      ]);

      if ($response->getStatusCode() === Response::HTTP_OK || $response->getStatusCode() === Response::HTTP_CREATED) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return $data['clientId'] ?? NULL;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to generate client ID: @error', ['@error' => $e->getMessage()]);
    }

    return NULL;
  }

  /**
   * Validate a client ID.
   *
   * @param string $clientId
   *   The client ID to validate.
   *
   * @return bool
   *   TRUE if the client ID is valid, FALSE otherwise.
   */
  public function validateClientId(string $clientId): bool {
    try {
      $response = $this->httpClient->request('GET', $this->getServerUrl() . "/api/clients/{$clientId}");

      if ($response->getStatusCode() === Response::HTTP_OK) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return $data['isValid'] ?? FALSE;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to validate client ID: @error', ['@error' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Create a new channel.
   *
   * @param \Drupal\notification_server\DTO\ChannelDTO $channel
   *   The channel data.
   *
   * @return bool
   *   TRUE if channel was created successfully, FALSE otherwise.
   */
  public function createChannel(ChannelDTO $channel): bool {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . '/api/channels', [
        'json' => $channel->toArray(),
      ]);

      if ($response->getStatusCode() === Response::HTTP_CREATED) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return isset($data['channel']);
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to create channel: @error', ['@error' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Grant access to a channel for a client.
   *
   * @param string $channel
   *   The channel name.
   * @param string $clientId
   *   The client ID to grant access to.
   *
   * @return bool
   *   TRUE if access was granted successfully, FALSE otherwise.
   */
  public function grantChannelAccess(string $channel, string $clientId): bool {
    try {
      $response = $this->httpClient->request('POST', $this->getServerUrl() . "/api/channels/{$channel}/access/{$clientId}");

      if ($response->getStatusCode() === Response::HTTP_OK) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return $data['accessGranted'] ?? FALSE;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to grant channel access: @error', ['@error' => $e->getMessage()]);
    }

    return FALSE;
  }

  /**
   * Revoke access to a channel for a client.
   *
   * @param string $channel
   *   The channel name.
   * @param string $clientId
   *   The client ID to revoke access from.
   *
   * @return bool
   *   TRUE if access was revoked successfully, FALSE otherwise.
   */
  public function revokeChannelAccess(string $channel, string $clientId): bool {
    try {
      $response = $this->httpClient->request('DELETE', $this->getServerUrl() . "/api/channels/{$channel}/access/{$clientId}");

      if ($response->getStatusCode() === Response::HTTP_OK) {
        $data = json_decode($response->getBody()->getContents(), TRUE);
        return $data['accessRevoked'] ?? FALSE;
      }
    }
    catch (GuzzleException $e) {
      $this->logger->error('Failed to revoke channel access: @error', ['@error' => $e->getMessage()]);
    }
    return FALSE;
  }

  /**
   * Gets the WebSocket server URL.
   *
   * @return string
   *   The WebSocket server URL.
   */
  public function getWebsocketEndpoint() {
    $config = $this->configFactory->get('notification_server.settings');
    return $config->get('websocket_url');
  }

}
